part of redstone.rethinkdb;

class RethinkServices<T> extends Table {
  final Rethinkdb r = new Rethinkdb();
  final String tableName;
  Connection _conn;

  RethinkServices(String tableName):
    this.tableName = tableName,
    super(tableName);

  RethinkServices.fromConnection(String tableName, Connection _conn):
    this._conn = _conn,
    this.tableName = tableName,
    super(tableName);

  Connection get conn {
    var rConn = app.request.attributes.dbConn;;
    return
      _conn != null? _conn:
      rConn is RethinkConnection? rConn.conn:
      rConn;
  }
  Table get table => r.table(tableName);

  Insert insertType(T record, {options}) {
    return insert(encode(record), options);
  }

  Future<dynamic> insertNow(T record, {options, global_optargs}) async {
    var resp = await insertType(record, options: options).run(conn, global_optargs);

    List genKeys = resp['generated_keys'];
    int inserted = resp['inserted'];

    return inserted < 1? null:
      inserted == 1? genKeys.first:
      genKeys;
  }

  Future<T> getNow (String id, {global_optargs}) async
  {
    var record = await get(id).run(conn, global_optargs);
    return record != null? encode(record): null;
  }

  Update updateTyped (String id, T record, {options})
    => get(id).update(encode(record), options);

  Future updateNow (String id, T record, {options, global_optargs})
    => updateTyped(id, record, options: options).run(conn, global_optargs);

  Future deleteNow (String id, {options, global_optargs})
    => get(id).delete(options).run(conn, global_optargs);

  Future<T> findOne (condition) async {
    var result = await filter(condition).limit(1).run(conn);
    return result != null? decode(result, T): null;
  }
}
